@@ -114,7 +114,9 @@ pull_Index <- function(Data, maxage) {
 
 
 
-int_s_sel <- function(s_selectivity, nfleet = 1) {
+int_s_sel <- function(s_selectivity, nfleet, silent = FALSE) {
+  if(is.null(s_selectivity)) return(-4)
+
   s_sel <- suppressWarnings(as.numeric(s_selectivity)) # Numbers match fleets, otherwise see next lines
   s_sel[s_selectivity == "B"] <- -4
   s_sel[s_selectivity == "SSB"] <- -3
@@ -130,22 +132,72 @@ int_s_sel <- function(s_selectivity, nfleet = 1) {
   if(any(is.na(s_sel))) {
     stop("Character entries for s_selectivity (for indices) must be either: \"B\", \"SSB\", \"logistic\", \"dome\", or \"free\"", call. = FALSE)
   }
-
+  
+  if(!silent) {
+    nsurvey <- length(s_selectivity)
+    message("\nIndex selectivity setup:")
+    for(sur in 1:nsurvey) {
+      if(s_sel[sur] > 0) {
+        sout <- paste("fishery fleet", s_sel[sur])
+      } else {
+        sout <- switch(s_sel[sur] %>% as.character(),
+                       "-4" = "total biomass",
+                       "-3" = "spawning biomass",
+                       "-2" = "individual parameters at age (free)",
+                       "-1" = "logistic function",
+                       "0" = "dome function")
+      }
+      message("Index ", sur, ": ", sout)
+    }
+  }
+  
   return(s_sel)
 }
 
-int_sel <- function(selectivity) {
+
+int_sel <- function(selectivity, RCMdata, silent = FALSE) {
   sel <- suppressWarnings(as.numeric(selectivity))
   sel[selectivity == "free"] <- -2
   sel[selectivity == "logistic"] <- -1
   sel[selectivity == "dome"] <- 0
-
+  
   if(any(is.na(sel))) {
-    stop("Character entries for s_selectivity (for fleets) must be either: \"logistic\", \"dome\", or \"free\"", call. = FALSE)
+    stop("Character entries for selectivity (for fleets) must be either: \"logistic\", \"dome\", or \"free\"", call. = FALSE)
   }
+  
+  if(!silent && !missing(RCMdata)) {
+    message("\nFishery selectivity setup:")
+    Yr <- RCMdata@Misc$CurrentYr - RCMdata@Misc$nyears:1 + 1
+    no_blocks <- apply(RCMdata@sel_block, 2, function(x) length(unique(x)) == 1) %>% all()
+    for(bb in 1:length(sel)) {
+      fout <- switch(sel[bb] %>% as.character(),
+                     "-2" = "individual parameters at age (free)",
+                     "-1" = "logistic function",
+                     "0" = "dome function")
+      if(no_blocks) {
+        message("Fleet ", bb, ": ", fout)
+      } else {
+        fleet <- lapply(1:ncol(RCMdata@sel_block), function(ff) {
+          y <- Yr[RCMdata@sel_block[, ff] == bb]
+          if(length(y)) {
+            if(all(diff(y) == 1)) {
+              paste0(ff, " (", range(y) %>% paste(collapse = "-"), ")")
+            } else {
+              paste0(ff, " (", range(y) %>% paste(collapse = "-"), ", with gaps)")
+            }
+          } else {
+            NULL
+          }
+        })
+        message("Block ", bb, " (", fout, ") assigned to fishery:\n", do.call(c, fleet) %>% paste(collapse = "\n"))
+      }
+    }
+  }
+  
   return(sel)
 }
 
+
 make_LWT <- function(LWT, nfleet, nsurvey) {
 
   if(is.null(LWT$Chist)) {
@@ -370,6 +422,10 @@ check_RCMdata <- function(RCMdata, OM, condition = c("catch", "catch2", "effort"
       if(nrow(RCMdata@CAA_ESS) != RCMdata@Misc$nyears) stop("Number of rows of CAA_ESS matrix does not equal nyears (", RCMdata@Misc$nyears, "). NAs are acceptable.", call. = FALSE)
       if(ncol(RCMdata@CAA_ESS) != RCMdata@Misc$nfleet) stop("Number of columns of CAA_ESS matrix does not equal nfleet (", RCMdata@Misc$nfleet, "). NAs are acceptable.", call. = FALSE)
     } else stop("CAA_ESS is neither a vector nor a matrix.", call. = FALSE)
+    
+    # Check if CAA_ESS > 0 if there are no data
+    RCMdata@CAA_ESS[apply(RCMdata@CAA, c(1, 3), sum, na.rm = TRUE) == 0] <- 0
+    
   } else {
     RCMdata@CAA <- array(0, c(RCMdata@Misc$nyears, OM@maxage + 1, RCMdata@Misc$nfleet))
     RCMdata@CAA_ESS <- matrix(0, RCMdata@Misc$nyears, RCMdata@Misc$nfleet)
@@ -407,6 +463,9 @@ check_RCMdata <- function(RCMdata, OM, condition = c("catch", "catch2", "effort"
       if(nrow(RCMdata@CAL_ESS) != RCMdata@Misc$nyears) stop("Number of rows of CAL_ESS matrix does not equal nyears (", RCMdata@Misc$nyears, "). NAs are acceptable.", call. = FALSE)
       if(ncol(RCMdata@CAL_ESS) != RCMdata@Misc$nfleet) stop("Number of columns of CAL_ESS matrix does not equal nfleet (", RCMdata@Misc$nfleet, "). NAs are acceptable.", call. = FALSE)
     } else stop("CAL_ESS is neither a vector nor a matrix.", call. = FALSE)
+    
+    # Check if CAL_ESS > 0 if there are no data
+    RCMdata@CAL_ESS[apply(RCMdata@CAL, c(1, 3), sum, na.rm = TRUE) == 0] <- 0
   } else {
     RCMdata@CAL_ESS <- matrix(0, RCMdata@Misc$nyears, RCMdata@Misc$nfleet)
   }
@@ -498,6 +557,9 @@ check_RCMdata <- function(RCMdata, OM, condition = c("catch", "catch2", "effort"
       if(nrow(RCMdata@IAA_ESS) != RCMdata@Misc$nyears) stop("Number of rows of IAA_ESS matrix does not equal nyears (", RCMdata@Misc$nyears, "). NAs are acceptable.", call. = FALSE)
       if(ncol(RCMdata@IAA_ESS) != RCMdata@Misc$nsurvey) stop("Number of columns of IAA_ESS matrix does not equal nsurvey (", RCMdata@Misc$nsurvey, "). NAs are acceptable.", call. = FALSE)
     } else stop("IAA_ESS is neither a vector nor a matrix.", call. = FALSE)
+    
+    # Check if IAA_ESS > 0 if there are no data
+    RCMdata@IAA_ESS[apply(RCMdata@IAA, c(1, 3), sum, na.rm = TRUE) == 0] <- 0
   } else {
     RCMdata@IAA <- array(0, c(RCMdata@Misc$nyears, OM@maxage + 1, ncol(RCMdata@Index)))
     RCMdata@IAA_ESS <- array(0, dim(RCMdata@Index))
@@ -525,6 +587,8 @@ check_RCMdata <- function(RCMdata, OM, condition = c("catch", "catch2", "effort"
       if(ncol(RCMdata@IAL_ESS) != RCMdata@Misc$nsurvey) stop("Number of columns of IAL_ESS matrix does not equal nfleet (", RCMdata@Misc$nsurvey, "). NAs are acceptable.", call. = FALSE)
     } else stop("IAL_ESS is neither a vector nor a matrix.", call. = FALSE)
     
+    # Check if IAL_ESS > 0 if there are no data
+    RCMdata@IAL_ESS[apply(RCMdata@IAL, c(1, 3), sum, na.rm = TRUE) == 0] <- 0
   } else {
     RCMdata@IAL_ESS <- array(0, dim(RCMdata@Index))
   }
@@ -603,6 +667,7 @@ check_RCMdata <- function(RCMdata, OM, condition = c("catch", "catch2", "effort"
     }
   }
   RCMdata@Misc$nsel_block <- as.numeric(RCMdata@sel_block) %>% unique() %>% length()
+  RCMdata@Misc$CurrentYr <- OM@CurrentYr
 
   return(list(RCMdata = RCMdata, OM = OM, StockPars = StockPars, ObsPars = ObsPars, FleetPars = FleetPars))
 }
