@@ -1,6 +1,6 @@
 
 RCM_est <- function(x = 1, RCMdata, selectivity, s_selectivity, LWT = list(),
-                    comp_like = c("multinomial", "lognormal"), prior = list(),
+                    comp_like = c("multinomial", "lognormal", "mvlogistic", "dirmult1", "dirmult2"), prior = list(),
                     max_F = 3, integrate = FALSE, StockPars, ObsPars, FleetPars, mean_fit = FALSE,
                     control = list(iter.max = 2e+05, eval.max = 4e+05), inner.control = list(maxit = 1e3), dots = list()) {
   
@@ -14,7 +14,7 @@ RCM_est <- function(x = 1, RCMdata, selectivity, s_selectivity, LWT = list(),
     }
   }
   
-  TMB_params <- RCM_est_params(x, RCMdata, selectivity, s_selectivity, prior, StockPars, FleetPars, dots)
+  TMB_params <- RCM_est_params(x, RCMdata, selectivity, s_selectivity, prior, LWT, comp_like, StockPars, FleetPars, dots)
   TMB_data <- RCM_est_data(x, RCMdata, selectivity, s_selectivity, LWT, comp_like, prior, max_F, 
                            StockPars, ObsPars, FleetPars, mean_fit, TMB_params$map, dots)
   
@@ -87,7 +87,7 @@ RCM_est_data <- function(x, RCMdata, selectivity, s_selectivity, LWT = list(), c
     if(RCMdata@Misc$condition == "effort") {
       StockPars$R0 <- mean_vector(StockPars$R0)
       if(!is.null(dots$OMeff) && dots$OMeff) {
-        FleetPars$Find <- apply(FleetPars$Find, 2, mean) %>% matrix(length(StockPars$R0), data$nyears, byrow = TRUE)
+        FleetPars$Find <- apply(FleetPars$Find, 2, mean) %>% matrix(length(StockPars$R0), nyears, byrow = TRUE)
       }
     }
     
@@ -139,7 +139,7 @@ RCM_est_data <- function(x, RCMdata, selectivity, s_selectivity, LWT = list(), c
                    CAA_hist = RCMdata@CAA, CAA_n = RCMdata@CAA_ESS,
                    CAL_hist = RCMdata@CAL, CAL_n = RCMdata@CAL_ESS,
                    IAA_hist = RCMdata@IAA, IAA_n = RCMdata@IAA_ESS,
-                   IAL_hist = RCMdata@IAL, IAL_n = RCMdata@IAL_ESS, 
+                   IAL_hist = RCMdata@IAL, IAL_n = RCMdata@IAL_ESS,
                    lbin = RCMdata@Misc$lbin, lbinmid = RCMdata@Misc$lbinmid, 
                    msize = RCMdata@MS, msize_type = RCMdata@MS_type,
                    sel_block = rbind(RCMdata@sel_block, RCMdata@sel_block[nyears, ]), 
@@ -184,7 +184,7 @@ RCM_est_data <- function(x, RCMdata, selectivity, s_selectivity, LWT = list(), c
   return(TMB_data)
 }
 
-RCM_est_params <- function(x, RCMdata, selectivity, s_selectivity, prior = list(),
+RCM_est_params <- function(x, RCMdata, selectivity, s_selectivity, prior = list(), LWT = list(), comp_like,
                            StockPars, FleetPars, dots = list()) {
   
   SR_type <- ifelse(StockPars$SRrel[x] == 1, "BH", "Ricker")
@@ -348,7 +348,8 @@ RCM_est_params <- function(x, RCMdata, selectivity, s_selectivity, prior = list(
                      log_q_effort = rep(log(0.1), nfleet), log_F_dev = matrix(0, nyears, nfleet),
                      log_F_equilibrium = rep(log(0.05), nfleet),
                      log_CV_msize = log(RCMdata@MS_cv), log_tau = log(StockPars$procsd[x]),
-                     log_early_rec_dev = log_early_rec_dev, log_rec_dev = log_rec_dev)
+                     log_early_rec_dev = log_early_rec_dev, log_rec_dev = log_rec_dev,
+                     log_compf = matrix(0, nfleet, 2), log_compi = matrix(0, nsurvey, 2))
   
   if(RCMdata@Misc$condition == "catch") {
     TMB_params$log_F_dev[as.integer(0.5 * nyears) + 1, ] <- log(0.5 * mean(StockPars$M_ageArray[x, , nyears]))
@@ -404,6 +405,33 @@ RCM_est_params <- function(x, RCMdata, selectivity, s_selectivity, prior = list(
     }
   }
   
+  map$log_compf <- local({
+    mapf <- matrix(NA, nfleet, 2)
+    
+    if(comp_like %in% c("dirmult1", "dirmult2")) {
+      CAAv <- sapply(1:nfleet, function(ff) LWT$CAA[ff] > 0 && sum(RCMdata@CAA_ESS[, ff]) > 0)
+      CALv <- sapply(1:nfleet, function(ff) LWT$CAL[ff] > 0 && sum(RCMdata@CAL_ESS[, ff]) > 0)
+      
+      est <- cbind(CAAv, CALv)
+      mapf[est] <- 1:sum(est)
+    }
+    
+    factor(mapf)
+  })
+  
+  map$log_compi <- local({
+    mapi <- matrix(NA, nsurvey, 2)
+    
+    if(comp_like %in% c("dirmult1", "dirmult2")) {
+      IAAv <- sapply(1:nsurvey, function(sur) LWT$IAA[sur] > 0 && sum(RCMdata@IAA_ESS[, sur]) > 0)
+      IALv <- sapply(1:nsurvey, function(sur) LWT$IAL[sur] > 0 && sum(RCMdata@IAL_ESS[, sur]) > 0)
+      
+      est <- cbind(IAAv, IALv)
+      mapi[est] <- 1:sum(est)
+    }
+    factor(mapi)
+  })
+  
   list(params = TMB_params, map = map)
 }
 
@@ -529,6 +557,33 @@ RCM_posthoc_adjust <- function(report, obj, par = obj$env$last.par.best, dynamic
     report$ivul_len <- get_ivul_len(report, data$ivul_type, lmid, data$Linf)
   }
   if(dynamic_SSB0) report$dynamic_SSB0 <- RCM_dynamic_SSB0(obj, par)
+  
+  if(data$comp_like == "mvlogistic") {
+    
+    CAAmv <- calc_mvlogistic_loglike(obs = data$CAA_hist, pred = report$CAApred, 
+                                     LWT = data$LWT_fleet[, 3], nllv = report$nll_fleet[1, , 3])
+    CALmv <- calc_mvlogistic_loglike(obs = data$CAL_hist, pred = report$CALpred, 
+                                     LWT = data$LWT_fleet[, 4], nllv = report$nll_fleet[1, , 4])
+    
+    IAAmv <- calc_mvlogistic_loglike(obs = data$IAA_hist, pred = report$IAApred, 
+                                     LWT = data$LWT_index[, 2], nllv = report$nll_index[1, , 2])
+    
+    if(is.null(report$IALpred)) {
+      IALmv <- list(tau = rep(NaN, data$nsurvey), nll = matrix(0, data$n_y, data$nsurvey))
+    } else {
+      IALmv <- calc_mvlogistic_loglike(obs = data$IAL_hist, pred = report$IALpred, 
+                                       LWT = data$LWT_index[, 3], nllv = report$nll_index[1, , 3])
+    }
+    
+    report$nll_fleet[, 1:data$nfleet, 3] <- CAAmv$nll
+    report$nll_fleet[, 1:data$nfleet, 4] <- CALmv$nll
+    
+    report$nll_index[, 1:data$nsurvey, 2] <- IAAmv$nll
+    report$nll_index[, 1:data$nsurvey, 3] <- IALmv$nll
+    
+    report$compf <- cbind(CAAmv$tau, CALmv$tau)
+    report$compi <- cbind(IAAmv$tau, IALmv$tau)
+  }
   return(report)
 }
 
@@ -603,3 +658,60 @@ RCM_SPR <- function(F_at_age, M, mat, wt, N_at_age, R, R_early, equilibrium = TR
   SPR <- SSPR_F/SSPR_0
   return(SPR)
 }
+
+
+dmvlogistic <- function(x, p, sd, xmin = 1e-8, log = FALSE) {
+  resid <- log(x[x > xmin]) - log(p[x > xmin])
+  if(!length(resid)) stop("Density function can not be calculated from x")
+  accum <- log(sum(x[x <= xmin])) - log(sum(p[x <= xmin]))
+  
+  eta <- c(resid, accum) - mean(c(resid, accum))
+  
+  A <- length(eta)
+  
+  # No normalizing constants!
+  log_like <- -(A-1) * log(sd) - 0.5 * sum(eta^2)/sd/sd
+  if(log) {
+    log_like
+  } else {
+    exp(log_like)
+  }
+}
+
+
+calc_mvlogistic_loglike <- function(obs, pred, nllv, LWT, obsmin = 1e-8) {
+  nf <- dim(obs)[3]
+  n_y <- dim(obs)[1]
+  
+  vars <- lapply(1:nf, function(ff) {
+    sum_count <- sapply(1:n_y, function(y) {
+      if(sum(obs[y, , ff]) > 0 && LWT[ff] > 0) {
+        A <- sum(obs[y, , ff] > obsmin)
+        if(any(obs[y, , ff] <= obsmin)) A <- A + 1
+        return(A-1)
+      } else {
+        return(0)
+      }
+    }) %>% sum()
+    
+    #tau2 <- exp((-nllv[ff] + 0.5 * sum_count)/(-0.5 * sum_count))
+    tau2 <- exp(nllv[ff]/(0.5 * sum_count) - 1)
+    tau <- sqrt(tau2)
+    
+    log_like <- sapply(1:n_y, function(y) {
+      if(sum(obs[y, , ff]) > 0 && LWT[ff] > 0) {
+        log_like <- dmvlogistic(x = obs[y, , ff], p = pred[y, , ff]/sum(pred[y, , ff]), 
+                                sd = tau, xmin = obsmin, log = TRUE)
+      } else {
+        log_like <- 0
+      }
+      return(log_like)
+    })
+    
+    list(tau = tau, log_like = log_like)
+  })
+  
+  list(tau = sapply(vars, getElement, "tau"),
+       nll = -1 * sapply(vars, getElement, "log_like"))
+}
+
