@@ -91,13 +91,12 @@ make_interim_MP <- function(.Assess = "SCA", .HCR = "HCR_MSY", AddInd = "VB", as
     if(run_assessment) {
       do_Assessment <- .(Assess_call)
       
-      if(diagnostic != "none") {
-        Assess_diag_output <- Assess_diagnostic(x, Data, do_Assessment, include_assessment = .(diagnostic == "full"))
-      }
-      
       if(do_Assessment@conv) { # Assessment converged. Run the HCR and report parameters for future interim procedure
         Rec <- .(HCR_call)
-        if(diagnostic != "none") Rec@Misc$diagnostic <- Assess_diag_output$diagnostic
+        
+        if(diagnostic != "none") {
+          Rec@Misc <- Assess_diagnostic(x, Data, do_Assessment, include_assessment = .(diagnostic == "full"))
+        }
         if(!is.null(do_Assessment@info$Misc)) Rec@Misc <- c(Rec@Misc, do_Assessment@info$Misc)
         
         Rec@Misc$interim <- list(Cref = Rec@TAC, # A vector of nsim reps
@@ -119,19 +118,22 @@ make_interim_MP <- function(.Assess = "SCA", .HCR = "HCR_MSY", AddInd = "VB", as
           Rec <- new("Rec")
           Rec@TAC <- rep(NA_real_, reps)
           
-          if(diagnostic != "none") Rec@Misc$diagnostic <- Assess_diag_output$diagnostic
+          if(diagnostic != "none") {
+            Rec@Misc <- Assess_diagnostic(x, Data, do_Assessment, include_assessment = .(diagnostic == "full"))
+          }
           if(!is.null(do_Assessment@info$Misc)) Rec@Misc <- c(Rec@Misc, do_Assessment@info$Misc)
           
-          Rec@Misc$interim <- list(Cref = NA_real_, Iref = NA_real_, next_assess_yr = next_assess_yr)
+          Rec@Misc$interim <- list(Cref = NA_real_, Iref = NA_real_, next_assess_yr = next_assess_yr, s = 0)
           return(Rec)
           
         } else { # There should be a previous assessment to continue the interim procedure for the current year
           run_interim <- TRUE
         }
       }
+      
     }
     
-    if(!run_assessment || run_interim) {
+    if(run_interim) {
       Cref <- Data@Misc[[x]]$interim$Cref
       Iref <- Data@Misc[[x]]$interim$Iref
       I_y <- switch(type,
@@ -161,11 +163,16 @@ make_interim_MP <- function(.Assess = "SCA", .HCR = "HCR_MSY", AddInd = "VB", as
       }
       
       Rec <- new("Rec")
-      Rec@TAC <- Cref * (I_y + b * s)/(Iref + b * s)
+      TAC <- Cref * (I_y + b * s)/(Iref + b * s)
+      if(is.null(TAC)) TAC <- NA_real_
+      Rec@TAC <- TAC
       
-      Rec@Misc <- Data@Misc[[x]]
       if(exists("next_assess_yr", inherits = FALSE)) Rec@Misc$interim$next_assess_yr <- next_assess_yr
-      if(exists("Assess_diag_output", inherits = FALSE)) Rec@Misc$diagnostic <- Assess_diag_output$diagnostic
+      if(exists("do_Assessment", inherits = FALSE) && diagnostic != "none") { # If we have an assessment
+        Rec@Misc <- Assess_diagnostic(x, Data, do_Assessment, include_assessment = .(diagnostic == "full"))
+        if(!is.null(do_Assessment@info$Misc)) Rec@Misc <- c(Rec@Misc, do_Assessment@info$Misc)
+      }
+      Rec@Misc <- c(Rec@Misc, Data@Misc[[x]][setdiff(names(Data@Misc[[x]]), names(Rec@Misc))]) # Only add items that are missing from Rec@Misc
     }
     return(Rec)
   })
@@ -180,3 +187,93 @@ make_interim_MP <- function(.Assess = "SCA", .HCR = "HCR_MSY", AddInd = "VB", as
   return(structure(custom_MP, class = "MP"))
 }
 
+
+#' @rdname make_MP
+#' @param Ftarget An expression that the MP will evaluate to identify the F used in the projection. See \link{projection} and example.
+#' @param proj_args Additional arguments for \link{projection}.
+#' @aliases make_projection_MP
+#' @export
+make_projection_MP <- function(.Assess = "SCA", .HCR = "HCR_MSY", assessment_interval = 5, 
+                               Ftarget = expression(Assessment@FMSY),
+                               proj_args = list(process_error = 1, p_sim = 1), 
+                               diagnostic = c("min", "full", "none"), ...) {
+  diagnostic <- match.arg(diagnostic)
+  
+  if(is.character(.Assess)) {
+    Assess_char <- .Assess
+    .Assess <- as.symbol(.Assess)
+  } else {
+    .Assess <- substitute(.Assess)
+    Assess_char <- as.character(.Assess)
+  }
+  #if(is.character(.HCR)) {
+  #  .HCR <- as.symbol(.HCR)
+  #} else {
+  #  .HCR <- substitute(.HCR)
+  #}
+  if(!inherits(eval(.Assess), "Assess")) {
+    stop(paste(.Assess, "does not belong to class 'Assess'. Use: avail('Assess') to find eligible objects."))
+  }
+  #if(!inherits(eval(.HCR), "HCR")) {
+  #  stop(paste(.HCR, "does not belong to class 'HCR.' Use: avail('HCR') to find eligible objects."))
+  #}
+  
+  dots <- list(...)
+  dots_in_Assess <- dots[match(names(formals(eval(.Assess))), names(dots), nomatch = 0)]
+  #dots_in_HCR <- dots[match(names(formals(eval(.HCR))), names(dots), nomatch = 0)]
+  
+  Assess_call <- as.call(c(.Assess, x = quote(x), Data = quote(Data), dots_in_Assess))
+  #HCR_call <- as.call(c(.HCR, Assessment = quote(Assessment), reps = quote(reps), dots_in_HCR))
+  
+  proj_args <- proj_args[match(names(formals(projection)), names(proj_args), nomatch = 0)]
+  proj_args$p_years <- assessment_interval + 1
+  
+  proj_call <- as.call(c(substitute(projection), Assessment = quote(Assessment), Ftarget = quote(eval(Ftarget)), proj_args))
+  
+  MP_body <- bquote({
+    dependencies <- .(get_dependencies(Assess_char, dots))
+    
+    ny <- length(Data@Year)
+    Current_Yr <- Data@Year[ny]
+    first_assess_yr <- Current_Yr == Data@LHYear
+    run_assessment <- first_assess_yr || Current_Yr == Data@Misc[[x]]$proj$next_assess_yr
+    get_projection <- !run_assessment
+    
+    Rec <- new("Rec")
+    if(run_assessment) {
+      Assessment <- .(Assess_call)
+      if(diagnostic != "none") {
+        Rec@Misc <- Assess_diagnostic(x, Data, Assessment, include_assessment = .(diagnostic == "full"))
+      }
+      if(!is.null(Assessment@info$Misc)) Rec@Misc <- c(Rec@Misc, Assessment@info$Misc)
+      
+      if(Assessment@conv) { # Assessment converged. Run the projection and save future TACs
+        Ftarget <- eval(Ftarget)
+        do_project <- .(proj_call)
+        
+        Rec@Misc$proj <- list(TACvec = apply(do_project@Catch, 2, median) %>% 
+                                structure(names = Current_Yr + 1 + 0:assessment_interval), # A vector of future TACs
+                              next_assess_yr = Current_Yr + assessment_interval)
+        Rec@TAC <- rep(Rec@Misc$proj$TACvec[1], reps)
+        
+      } else { # Assessment did not converge. Try the assessment again next year. TAC defaults to most recent TAC
+        Rec@Misc$proj <- list(TACvec = NA_real_, next_assess_yr = Current_Yr + 1)
+        Rec@TAC <- rep(NA_real_, reps)
+      }
+    }
+    
+    if(get_projection) {
+      Rec@Misc <- Data@Misc[[x]]
+      Rec@TAC <- rep(Data@Misc[[x]]$proj$TACvec[as.character(Current_Yr + 1)], reps)
+    }
+    return(Rec)
+  })
+  
+  custom_MP <- eval(call("function", as.pairlist(alist(x = 1, Data = , reps = 1)), MP_body))
+  formals(custom_MP)$Ftarget <- Ftarget
+  formals(custom_MP)$assessment_interval <- assessment_interval
+  formals(custom_MP)$diagnostic <- diagnostic
+  
+  return(structure(custom_MP, class = "MP"))
+}
+
